from django.conf.urls import patterns, include
from geonode.urls import urlpatterns as geonode_url_patterns
from maploom.geonode.urls import urlpatterns as maploom_urls

urlpatterns = patterns('',
                       (r'^file-service/', include('geoshape.file_service.urls')),
                       (r'^proxy/', 'geoshape.views.proxy'),
                       )

urlpatterns += geonode_url_patterns
urlpatterns += maploom_urls



from django.conf.urls import patterns, url
from views import GetImage, UploadImage

urlpatterns = patterns('',
                       url(r'upload/?$', UploadImage.as_view(), name='file_service_upload'),
                       url(r'^(?P<key>[-\w\d\.]+?)$', GetImage.as_view(), name='file_service'),
                       )



import logging
from django.conf import settings
from django.http import HttpResponse
from django.http.request import validate_host
from django.utils.http import is_safe_url
from httplib import HTTPConnection
from urlparse import urlsplit
from geonode.geoserver.helpers import ogc_server_settings

logger = logging.getLogger(__name__)


def proxy(request):
    PROXY_ALLOWED_HOSTS = getattr(settings, 'PROXY_ALLOWED_HOSTS', ())
    hostname = (ogc_server_settings.hostname,) if ogc_server_settings else ()
    PROXY_ALLOWED_HOSTS += hostname

    if 'url' not in request.GET:
        return HttpResponse("The proxy service requires a URL-encoded URL as a parameter.",
                            status=400,
                            content_type="text/plain"
                            )

    raw_url = request.GET['url']
    url = urlsplit(raw_url)

    locator = url.path
    if url.query != "":
        locator += '?' + url.query
    if url.fragment != "":
        locator += '#' + url.fragment

    logger.debug('Incoming headers: {0}'.format(request.META))

    if not settings.DEBUG:
        if not validate_host(url.hostname, PROXY_ALLOWED_HOSTS):
            return HttpResponse("DEBUG is set to False but the host of the path provided "
                                "to the proxy service is not in the "
                                "PROXY_ALLOWED_HOSTS setting.",
                                status=403,
                                content_type="text/plain"
                                )
    headers = {}

    if settings.SESSION_COOKIE_NAME in request.COOKIES and is_safe_url(url=raw_url, host=ogc_server_settings.netloc):
        headers["Cookie"] = request.META["HTTP_COOKIE"]

    if request.META.get('HTTP_AUTHORIZATION'):
        headers['AUTHORIZATION'] = request.META.get('HTTP_AUTHORIZATION')

    if request.method in ("POST", "PUT") and "CONTENT_TYPE" in request.META:
        headers["Content-Type"] = request.META["CONTENT_TYPE"]

    if request.META.get('HTTP_ACCEPT'):
        headers['ACCEPT'] = request.META['HTTP_ACCEPT']

    logger.debug('Outgoing request method: {0}'.format(request.method))
    logger.debug('Outgoing request locator: {0}{1}'.format(url.hostname, locator))
    logger.debug('Outgoing request headers: {0}'.format(headers))

    conn = HTTPConnection(url.hostname, url.port)
    conn.request(request.method, locator, request.raw_post_data, headers)
    result = conn.getresponse()

    logger.debug('Response headers: {0}'.format(result.getheaders()))
    logger.debug('Response status: {0}'.format(result.status))

    response = HttpResponse(result.read(),
                            status=result.status,
                            content_type=result.getheader("Content-Type", "text/plain"),
                            )

    if result.getheader('www-authenticate'):
        response['www-authenticate'] = "GeoNode"

    return response



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import Client, TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User


class SimpleTest(TestCase):

    def setUp(self):
        user, created = User.objects.get_or_create(username='test_admin',
                                                   password='admin',
                                                   is_active=True)

        user.set_password('admin')
        user.save()

        self.user = user

    def test_index(self):
        """
        Test for the main view.
        """

        c = Client()
        response = c.get(reverse('file_service', kwargs=dict(key='test')))

        # Non-authenticated user should return 401
        self.assertEqual(response.status_code, 401)

        logged_in = c.login(username='test_admin', password='admin')
        self.assertTrue(logged_in)

        # An authenticated user should return a 200
        response = c.get(reverse('file_service', kwargs=dict(key='test')))
        self.assertEqual(response.status_code, 200)

        c.logout()

        # A client should get a 200 response with basic authorization
        headers = dict(HTTP_AUTHORIZATION="basic dGVzdF9hZG1pbjphZG1pbg==")
        response = c.get(reverse('file_service', kwargs=dict(key='test')), **headers)
        self.assertEqual(response.status_code, 200)

        # A client should get a 401 response with incorrect basic authorization
        headers = dict(HTTP_AUTHORIZATION="basic dGasdasdsabg==")
        response = c.get(reverse('file_service', kwargs=dict(key='test')), **headers)
        self.assertEqual(response.status_code, 401)



# -*- coding: utf-8 -*-

import geonode
import os

from geonode.settings import *  # noqa

SITENAME = 'GeoSHAPE'

# Defines the directory that contains the settings file as the PROJECT_ROOT
# It is used for relative settings elsewhere.
PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))
GEONODE_ROOT = os.path.abspath(os.path.dirname(geonode.__file__))

# Setting debug to true makes Django serve static media and
# present pretty error pages.
ALLOWED_HOSTS = ()
CACHE_TIME = 0
CLASSIFICATION_BANNER_ENABLED = False
DEBUG = TEMPLATE_DEBUG = True
GEOGIT_DATASTORE_NAME = 'DEFAULT_NAME'
LOCKDOWN_GEONODE = True
REGISTRATION_OPEN = False
SOCIAL_BUTTONS = False
MODIFY_TOPICCATEGORY = False
# Set to True to load non-minified versions of (static) client dependencies
DEBUG_STATIC = False

# This is needed for integration tests, they require
# geonode to be listening for GeoServer auth requests.


# Defines settings for development
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_ROOT, 'development.db'),
    }
}

# OGC (WMS/WFS/WCS) Server Settings
OGC_SERVER = {
    'default': {
        'BACKEND': 'geonode.geoserver',
        'LOCATION': 'http://localhost:8080/geoserver/',
        'PUBLIC_LOCATION': 'http://localhost:8080/geoserver/',
        'USER': 'admin',
        'PASSWORD': 'geoserver',
        'MAPFISH_PRINT_ENABLED': True,
        'PRINTNG_ENABLED': True,
        'GEONODE_SECURITY_ENABLED': True,
        'GEOGIT_ENABLED': True,
        'WMST_ENABLED': False,
        'BACKEND_WRITE_ENABLED': True,
        'WPS_ENABLED': False,
        # Set to name of database in DATABASES dictionary to enable
        'DATASTORE': '',
        'TIMEOUT': 10,  # The maximum time to wait for the server to respond.
    }
}

# Uploader Settings
UPLOADER = {
    'BACKEND': 'geonode.rest',
    'OPTIONS': {
        'TIME_ENABLED': False,
        'GEOGIT_ENABLED': True,
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en'

WSGI_APPLICATION = "geoshape.wsgi.application"

# Additional directories which hold static files
STATICFILES_DIRS = [
    os.path.join(PROJECT_ROOT, "static"),
    os.path.join(GEONODE_ROOT, "static"),
]

# Note that Django automatically includes the "templates" dir in all the
# INSTALLED_APPS, se there is no need to add maps/templates or admin/templates
TEMPLATE_DIRS = (
    os.path.join(PROJECT_ROOT, "templates"),
    os.path.join(GEONODE_ROOT, "templates"),
)

# Location of translation files
LOCALE_PATHS = (
    os.path.join(PROJECT_ROOT, "locale"),
    os.path.join(GEONODE_ROOT, "locale"),
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'x-#u&4x2k*$0-60fywnm5&^+&a!pd-ajrx(z@twth%i7^+oskh'

# Location of url mappings
ROOT_URLCONF = 'geoshape.urls'

MAX_DOCUMENT_SIZE = 2  # MB


INSTALLED_APPS = (
    'geoshape.file_service',
    'geoshape.core',
    'django_classification_banner',
    'maploom',
) + INSTALLED_APPS

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(name)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(message)s'},
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'null': {
            'level': 'ERROR',
            'class': 'django.utils.log.NullHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler',
        }
    },
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": "ERROR",
        },
        "geoshape": {
            "handlers": ["console"],
            "level": "DEBUG",
        },
        "geonode": {
            "handlers": ["console"],
            "level": "DEBUG",
        },

        "gsconfig.catalog": {
            "handlers": ["console"],
            "level": "ERROR",
        },
        "owslib": {
            "handlers": ["console"],
            "level": "ERROR",
        },
        "pycsw": {
            "handlers": ["console"],
            "level": "ERROR",
        },
        'south': {
            "handlers": ["console"],
            "level": "ERROR",
        },
    },
}

TEMPLATE_CONTEXT_PROCESSORS += (
    'django_classification_banner.context_processors.classification',
    'geoshape.core.context_processors.security_warnings'
)

# Add additional paths (as regular expressions) that don't require authentication.
AUTH_EXEMPT_URLS = ('/file-service/*', '/i18n/setlang/',)

if LOCKDOWN_GEONODE:
    MIDDLEWARE_CLASSES = MIDDLEWARE_CLASSES + ('geonode.security.middleware.LoginRequiredMiddleware',)

# Load more settings from a file called local_settings.py if it exists
try:
    from local_settings import *  # noqa
except ImportError:
    pass

MAP_BASELAYERS = [
    {
        "source": {
            "ptype": "gxp_wmscsource",
            "url": OGC_SERVER['default']['LOCATION'] + "wms",
            "restUrl": "/gs/rest",
            "name": "local geoserver"
        }
    },
    {
        "source": {"ptype": "gxp_osmsource", "name": "OpenStreetMap"},
        "type": "OpenLayers.Layer.OSM",
        "name": "mapnik",
        "title": "OpenStreetMap",
        "args": ["OpenStreetMap"],
        "visibility": True,
        "fixed": True,
        "group":"background"
    }
]



import httplib2
from django.contrib.auth import authenticate
from django.http import HttpResponse
from django.views.generic import View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from geonode.utils import _get_basic_auth_info
from logging import getLogger

logger = getLogger(__name__)


class BasicAuthView(View):
    """
    A mixin that requires the user to be logged in or logs in the user with basic auth, if the appropriate headers are
    present, before rendering the response.
    """

    unauthenticated_response = HttpResponse("Unauthorized, please authenticate.", status=401)

    def dispatch(self, *args, **kwargs):
        user = self.request.user
        logger.debug(self.request)
        if not user.is_authenticated() and self.request.META.get('HTTP_AUTHORIZATION'):
            try:
                logger.debug("User is not logged in, but the request has basic auth headers.  Attempting to log "
                             "the user in.")
                username, password = _get_basic_auth_info(self.request)
                user = authenticate(username=username, password=password)
                logger.debug("Able to log user in: {0}".format(user.is_authenticated()))

            except Exception, e:
                logger.debug(e)
                return self.unauthenticated_response

        return self.unauthenticated_response if not user.is_authenticated() else \
            super(BasicAuthView, self).dispatch(*args, **kwargs)


class GetImage(BasicAuthView):
    """
    Proxies GET requests to the file-service application requiring authentication along the way.
    """
    http_method_names = ['get']

    def get(self, request, *args, **kwargs):
        key = kwargs.get('key')
        redirect_url = "http://127.0.0.1:8080/file-service/services/document/download?blobKey={}".format(key)
        http = httplib2.Http()
        response, content = http.request(redirect_url, "GET")

        return HttpResponse(
            content=content,
            status=response.status,
            mimetype=response.get("content-type", "text/plain"))


class UploadImage(BasicAuthView):
    """
    Proxies POST requests to the file-service application requiring authentication along the way.
    """
    http_method_names = ['post']

    def post(self, *args, **kwargs):
        http = httplib2.Http()
        url = "http://127.0.0.1:8080/file-service/services/document/upload"
        headers = dict((header, value) for header, value in self.request.META.items()
                       if header.startswith('HTTP_'))

        headers['CONTENT-TYPE'] = self.request.META.get('CONTENT_TYPE', '')
        headers['HOST'] = '127.0.0.1'
        response, content = http.request(url, body=self.body, method='POST', headers=headers)

        return HttpResponse(content=content,
                            status=response.status,
                            mimetype=response.get("content-type", "text/plain"))

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        self.body = self.request.body
        return super(UploadImage, self).dispatch(*args, **kwargs)



import os
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

# Tell distutils not to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
walk_dir = 'geoshape'

excluded_folders = ['uploaded']

for dirpath, dirnames, filenames in os.walk(walk_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.') or dirname in excluded_folders: del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

install_requires = [
        "geoshape-geonode==1.0",
        "django-classification-banner>=0.1.4",
        "django-maploom==1.0",
        "psycopg2==2.4.5"
]

tests_requires = [
    "lxml==3.3.1",
]

docs_requires = [
    "sphinx-rtd-theme==0.1.5",
    "Sphinx==1.2.2",
]

setup(
    name="geoshape",
    version="1.0",
    author="LMN Solutions",
    author_email="rogue@lmnsolutions.com",
    description="geoshape, based on GeoNode",
    long_description=(read('README.rst')),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
    ],
    license="BSD",
    keywords="geoshape geonode django",
    url='https://github.com/ROGUE-JCTD/rogue_geonode',
    packages=packages,
    data_files=data_files,
    include_package_data=True,
    install_requires=install_requires,
    extras_require={
        'tests': install_requires + tests_requires,
        'docs':  docs_requires
    },
    zip_safe=False,
)
